/*
 * tic-tac-toe
 */

#include <iostream>
#include <vector>

using namespace std; 

enum Token {
	token_empty, 
	token_opponent, 
	token_self
};


void print_token( Token t ){
	switch( t ){
		case token_opponent : std::cout << "x"; break; 
		case token_self : std::cout << "o"; break; 
		default: 
		case token_empty : std::cout << " "; break; 
	}
}

const size_t max_x = 3;
const size_t max_y = 3;


struct Move{
	size_t x; 
	size_t y; 
	Token t;

	Move( size_t x, size_t y ) 
				: x(x), y(y) {
	}

	Move( size_t x, size_t y, Token t ) 
				: x(x), y(y), t(t){

	}

	Move( ){
		x = 0;
		y = 0;
		t = token_empty; 
	}

};


struct MoveAnalysis{
	Move m; 
	int weight; // the greater, the best the movement is
};


struct Board{

	Token board[max_x][max_y];

	Board(){
		for( auto y = 0; y < max_y; ++y )
			for( auto x = 0; x < max_x; ++x ){
				board[x][y] = token_empty;
			}
	}

	// copy constructor
	Board( const Board &b ){
		for( auto y = 0; y < max_y; ++y )
			for( auto x = 0; x < max_x; ++x ){
				board[x][y] = b.board[x][y];
			}
	}

	void do_movement( const Move &m ) {
		board[m.x][m.y] = m.t; 
	}

	void print(){
		print_dashed_line();
		for( auto y = 0; y < max_y; ++y ){

			for( auto x = 0; x < max_x; ++x ){
				std::cout << "| ";
				print_token( board[x][y] );
				std::cout << " ";
			}
			std::cout << "|" << std::endl;
			print_dashed_line();
		}
	}

	void print_dashed_line( ){
		std::cout << "+";
		for( auto y = 0; y < max_y; ++y ){
			std::cout << "---+";
		}
		std::cout << std::endl;
	}

	Token get( size_t x, size_t y ) const {
		return board[x][y];
	}
};

Move get_opponent_movement(){
	cout << "movement (x y): "; 
	Move m; 
	do{
		cin >> m.x >> m.y; cin.ignore();
	}while( m.x >= max_x || m.y >= max_y );
	return m;
}

std::vector<Move> get_possible_movements( const Board &board, const Token &turn ){
	std::vector<Move> move_list; 
	for( auto y = 0; y < max_y; ++y ){
		for( auto x = 0; x < max_x; ++x ){
			if( board.get(x, y) == token_empty )
				move_list.push_back( Move( x, y, turn ) ); 
		}
	}
	return move_list;
}


bool line_equal( const Board &board, size_t line, Token turn ){
	bool same_turn = true; 
	for( auto x = 0; x < max_x; ++x ){
		same_turn = same_turn && board.get( line, x ) == turn; 
	}
	return same_turn; 
}

bool col_equal( const Board &board, size_t col, Token turn ){
	bool same_turn = true; 
	for( auto y = 0; y < max_y; ++y ){
		same_turn = same_turn && board.get( y, col ) == turn; 
	}
	return same_turn; 
}

bool diag_equal( const Board &board, Token turn ){
	bool diag_equal = true; 
	for( size_t x = 0, y = 0; x < max_x && y < max_y; ++x, ++y ){
		diag_equal = diag_equal && board.get(x,y) == turn;
	}
	return diag_equal;
}

bool inverse_diag_equal( const Board &board, Token turn ){
	bool diag_equal = true; 
	for( size_t x = max_x-1, y = 0; x >= 0 && y < max_y; --x, ++y ){
		std::cout << "checking x = " << x << ", y = " << y << std::endl;
		diag_equal = diag_equal && board.get(x,y) == turn;
	}
	return diag_equal;
}




bool is_winner( const Board &board, Token turn ){
	bool is_winner_line = false; 
	for( auto y = 0; y < max_y; ++y ) 
		is_winner_line = is_winner_line || line_equal( board, y, turn ); 
	bool is_winner_col = false; 
	for( auto x = 0; x < max_x; ++x )
		is_winner_col = is_winner_col || col_equal( board, x, turn ); 
	bool is_diag_equal = diag_equal( board, turn ); 
	bool is_inverse_diag = inverse_diag_equal( board, turn ); 

	return is_winner_line || is_winner_col || is_diag_equal || is_inverse_diag;
}

Move get_best_movement( const Board &board, const Token &turn, MoveAnalysis movement_beign_analyzed ){
	
	if( is_winner( board, token_self ) ){
		movement_beign_analyzed.weight = 100; 
	}
	if( is_winner( board, token_opponent) ){
		movement_beign_analyzed.weight = 0; 
	}

	???
	std::vector<Move> possible_movements = get_possible_movements( board, turn );
	for( Move m : possible_movements ){
		Board possible_board( board ); 
		possible_board.do_movement( m );
		possible_board.print();
		
	}
}


void test_winner(){

	Board b;

	std::cout << "testing is_winner..." << std::endl;
	b.print();
	b.do_movement( Move( 0, 0, token_self ) );
	b.do_movement( Move( 0, 1, token_self ) );
	b.do_movement( Move( 0, 2, token_self ) );
	b.print();
	if( is_winner( b, token_self ) ){
		std::cout << "ok" << std::endl; 
	}else{
		std::cout << "fail" << std::endl;
	}
	std::cout << "is winner: " << is_winner( b, token_self ) << std::endl;

}

void test_winner2(){

	Board b;

	std::cout << "testing is_winner..." << std::endl;
	b.print();
	b.do_movement( Move( 1, 0, token_self ) );
	b.do_movement( Move( 1, 1, token_self ) );
	b.do_movement( Move( 1, 2, token_self ) );
	b.print();
	if( is_winner( b, token_self ) ){
		std::cout << "ok" << std::endl; 
	}else{
		std::cout << "fail" << std::endl;
	}
	std::cout << "is winner: " << is_winner( b, token_self ) << std::endl;

}

void test_winner3(){

	Board b;

	std::cout << "testing is_winner..." << std::endl;
	b.print();
	b.do_movement( Move( 0, 0, token_self ) );
	b.do_movement( Move( 1, 0, token_self ) );
	b.do_movement( Move( 2, 0, token_self ) );
	b.print();
	if( is_winner( b, token_self ) ){
		std::cout << "ok" << std::endl; 
	}else{
		std::cout << "fail" << std::endl;
	}
	std::cout << "is winner: " << is_winner( b, token_self ) << std::endl;

}


void test_winner4(){

	Board b;

	std::cout << "testing is_winner..." << std::endl;
	b.print();
	b.do_movement( Move( 2, 0, token_self ) );
	b.do_movement( Move( 1, 1, token_self ) );
	b.do_movement( Move( 0, 2, token_self ) );
	b.print();
	if( is_winner( b, token_self ) ){
		std::cout << "ok" << std::endl; 
	}else{
		std::cout << "fail" << std::endl;
	}
	std::cout << "is winner: " << is_winner( b, token_self ) << std::endl;

}

int main( int argc, char *argv[] ){
	
	cout << "tictactoe" << endl; 
	//Move m = get_opponent_movement();
	//cout << "movement is " << m.x << " " << m.y << endl;

}

